@@ -1,14 +1,35 @@
+/*
+Copyright 2018 Ron Triepels
+
+Licensed under the Apache License, Version 2.0 (the "License");
+you may not use this file except in compliance with the License.
+You may obtain a copy of the License at
+
+    http://www.apache.org/licenses/LICENSE-2.0
+
+Unless required by applicable law or agreed to in writing, software
+distributed under the License is distributed on an "AS IS" BASIS,
+WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+See the License for the specific language governing permissions and
+limitations under the License.
+*/
+
 #include <R.h>
 #include <stdlib.h>
 #include <Rinternals.h>
 #include <R_ext/Rdynload.h>
 
+/* fun.c functions */
 extern SEXP address(SEXP);
 extern SEXP bsum(SEXP, SEXP);
+
+/* cgraph.c functions */
 extern SEXP cgraph(SEXP, SEXP);
+extern SEXP cg_add_constant(SEXP, SEXP, SEXP);
+extern SEXP cg_add_input(SEXP, SEXP, SEXP);
 extern SEXP cg_add_operation(SEXP, SEXP, SEXP, SEXP, SEXP);
+extern SEXP cg_add_parameter(SEXP, SEXP, SEXP);
 extern SEXP cg_add_parms();
-extern SEXP cg_add_placeholder(SEXP, SEXP, SEXP, SEXP);
 extern SEXP cg_adj_mat(SEXP);
 extern SEXP cg_approx_grad(SEXP, SEXP, SEXP, SEXP, SEXP, SEXP);
 extern SEXP cg_gen_name(SEXP, SEXP);
@@ -20,9 +41,11 @@ static const R_CallMethodDef CallEntries[] = {
   {"address",            (DL_FUNC) &address,            1},
   {"bsum",               (DL_FUNC) &bsum,               2},
   {"cgraph",             (DL_FUNC) &cgraph,             2},
+  {"cg_add_constant",    (DL_FUNC) &cg_add_constant,    3},
+  {"cg_add_input",       (DL_FUNC) &cg_add_input,       3},
   {"cg_add_operation",   (DL_FUNC) &cg_add_operation,   5},
+  {"cg_add_parameter",   (DL_FUNC) &cg_add_parameter,   3},
   {"cg_add_parms",       (DL_FUNC) &cg_add_parms,       2},
-  {"cg_add_placeholder", (DL_FUNC) &cg_add_placeholder, 4},
   {"cg_adj_mat",         (DL_FUNC) &cg_adj_mat,         1},
   {"cg_approx_grad",     (DL_FUNC) &cg_approx_grad,     6},
   {"cg_gen_name",        (DL_FUNC) &cg_gen_name,        2},
