@@ -1,3 +1,19 @@
+/*
+Copyright 2018 Ron Triepels
+
+Licensed under the Apache License, Version 2.0 (the "License");
+you may not use this file except in compliance with the License.
+You may obtain a copy of the License at
+
+    http://www.apache.org/licenses/LICENSE-2.0
+
+Unless required by applicable law or agreed to in writing, software
+distributed under the License is distributed on an "AS IS" BASIS,
+WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+See the License for the specific language governing permissions and
+limitations under the License.
+*/
+
 #include <stdio.h>
 #include <stdlib.h>
 
@@ -16,7 +32,7 @@ stack stack_init(int maxSize)
 
   if(data == NULL)
   {
-    error("insufficient memory to initialize stack");
+    error("insufficient memory to allocate stack of %d elements", maxSize);
   }
 
   s.top = -1;
@@ -53,7 +69,7 @@ void stack_push(stack *s, int x)
 
     if(s->data == NULL)
     {
-      error("insufficient memory to initialize stack");
+      error("insufficient memory to reallocate stack of %d elements", 2 * s->maxSize);
     }
 
     s->maxSize *= 2;
